
#pragma once

// Fix an error, that occurs if a previousli included
// file uses '#define PAGE_SIZE ...' which then is used
// by the pre processor to replace the enum name in line
// 79 of <base/attached_ram_dataspace.h>
#undef PAGE_SIZE

#include <base/attached_ram_dataspace.h>
#include <base/env.h>
#include <tee/packet_allocator.h>
#include <tee_session/rpc_object.h>

namespace Tee {
	class Communication_buffers;
	class Session_component;
}


class Tee::Communication_buffers
{
	protected:

		Tee::Packet_allocator          _rx_packet_alloc;
		Genode::Attached_ram_dataspace _tx_ds, _rx_ds;

		Communication_buffers(Genode::Allocator      &rx_block_md_alloc,
		                      Genode::Ram_allocator  &ram,
		                      Genode::Region_map     &region_map,
		                      Genode::size_t          tx_size,
		                      Genode::size_t          rx_size,
		                      Genode::Cache_attribute cache_policy)
		:
			_rx_packet_alloc(&rx_block_md_alloc),
			_tx_ds(ram, region_map, tx_size, cache_policy),
			_rx_ds(ram, region_map, rx_size, cache_policy)
		{ }
};


class Tee::Session_component : Communication_buffers, public Session_rpc_object
{
	protected:

		Genode::Entrypoint &_ep;

		/**
		 * Sub-classes must implement this function, it is called upon all
		 * packet-stream signals.
		 */
		virtual void _handle_packet_stream() = 0;

		void _dispatch() { _handle_packet_stream(); }

		Genode::Signal_handler<Session_component> _packet_stream_dispatcher {
			_ep, *this, &Session_component::_dispatch };

		using Communication_buffers::_tx_ds;
		using Communication_buffers::_rx_ds;

	public:

		/**
		 * Constructor
		 *
		 * \param tx_buf_size        buffer size for tx channel
		 * \param rx_buf_size        buffer size for rx channel
		 * \param rx_block_md_alloc  backing store of the meta data of the
		 *                           rx block allocator
		 * \param env                Genode environment needed to access
		 *                           resources and open connections from
		 *                           within the Session_component
		 */
		Session_component(Genode::size_t const    tx_buf_size,
		                  Genode::size_t const    rx_buf_size,
		                  Genode::Cache_attribute cache_policy,
		                  Genode::Allocator      &rx_block_md_alloc,
		                  Genode::Env            &env)
		:
			Communication_buffers(rx_block_md_alloc, env.ram(), env.rm(),
			                      tx_buf_size, rx_buf_size, cache_policy),
			Session_rpc_object(env.rm(),
			                   _tx_ds.cap(),
			                   _rx_ds.cap(),
			                  &_rx_packet_alloc, env.ep().rpc_ep()),
			_ep(env.ep())
		{
			/* install data-flow signal handlers for both packet streams */
			_tx.sigh_ready_to_ack(_packet_stream_dispatcher);
			_tx.sigh_packet_avail(_packet_stream_dispatcher);
			_rx.sigh_ready_to_submit(_packet_stream_dispatcher);
			_rx.sigh_ack_avail(_packet_stream_dispatcher);
		}
};


